Package: bslib
Title: Custom 'Bootstrap' 'Sass' Themes for 'shiny' and 'rmarkdown'
Version: 0.4.1
Authors@R: c(
    person("Carson", "Sievert", role = c("aut", "cre"), email = "carson@rstudio.com", comment = c(ORCID = "0000-0002-4958-2844")),
    person("Joe", "Cheng", role = "aut", email = "joe@rstudio.com"),
    person(family = "RStudio", role = "cph"),
    person(family = "Bootstrap contributors", role = "ctb",
    comment = "Bootstrap library"),
    person(family = "Twitter, Inc", role = "cph",
    comment = "Bootstrap library"),
    person("Javi", "Aguilar", role = c("ctb", "cph"),
    comment = "Bootstrap colorpicker library"),
    person("Thomas", "Park", role = c("ctb", "cph"),
    comment = "Bootswatch library"),
    person(family = "PayPal", role = c("ctb", "cph"),
    comment = "Bootstrap accessibility plugin")
    )
Description: Simplifies custom 'CSS' styling of both 'shiny' and 'rmarkdown' via 'Bootstrap' 'Sass'. Supports both 'Bootstrap' 3 and 4 as well as their various 'Bootswatch' themes. An interactive widget is also provided for previewing themes in real time.
Depends: R (>= 2.10)
Imports: grDevices, htmltools (>= 0.5.2), jsonlite, sass (>= 0.4.0),
        jquerylib (>= 0.1.3), rlang, cachem, memoise (>= 2.0.1)
Suggests: shiny (>= 1.6.0), rmarkdown (>= 2.7), thematic, knitr,
        testthat, withr, rappdirs, curl, magrittr
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.2.1
Collate: 'bootswatch.R' 'bs-current-theme.R' 'bs-dependencies.R'
        'bs-global.R' 'bs-remove.R' 'bs-theme-layers.R' 'utils.R'
        'bs-theme-preview.R' 'bs-theme-update.R' 'bs-theme.R'
        'deprecated.R' 'files.R' 'imports.R' 'nav-items.R'
        'nav-update.R' 'navs-legacy.R' 'navs.R' 'onLoad.R' 'page.R'
        'precompiled.R' 'print.R' 'shiny-devmode.R' 'staticimports.R'
        'utils-shiny.R' 'utils-tags.R' 'version-default.R' 'versions.R'
URL: https://rstudio.github.io/bslib/, https://github.com/rstudio/bslib
BugReports: https://github.com/rstudio/bslib/issues
Config/testthat/edition: 3
Config/Needs/routine: desc, renv
Config/Needs/website: brio, dplyr, DT, glue, purrr, rprojroot,
        rstudio/quillt, stringr, tidyr
Config/Needs/deploy: BH, DT, ggplot2, hexbin, lattice, reactable,
        rprojroot, rsconnect
NeedsCompilation: no
Packaged: 2022-11-02 20:21:31 UTC; cpsievert
Author: Carson Sievert [aut, cre] (<https://orcid.org/0000-0002-4958-2844>),
  Joe Cheng [aut],
  RStudio [cph],
  Bootstrap contributors [ctb] (Bootstrap library),
  Twitter, Inc [cph] (Bootstrap library),
  Javi Aguilar [ctb, cph] (Bootstrap colorpicker library),
  Thomas Park [ctb, cph] (Bootswatch library),
  PayPal [ctb, cph] (Bootstrap accessibility plugin)
Maintainer: Carson Sievert <carson@rstudio.com>
Repository: CRAN
Date/Publication: 2022-11-02 21:10:10 UTC
