utils::globalVariables("!!")

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import htmltools
#' @import sass
#' @importFrom utils modifyList packageVersion download.file URLencode getFromNamespace head
#' @importFrom stats setNames na.omit
#' @importFrom grDevices col2rgb
#' @importFrom tools file_path_sans_ext
#' @importFrom jquerylib jquery_core
#' @importFrom jsonlite fromJSON
#' @importFrom rlang := list2 abort
## usethis namespace: end
NULL
