# input_submit_textarea() markup snapshots Code input_submit_textarea("test") Output
    ---

        Code
          input_submit_textarea("test", label = "Enter text", placeholder = "Type here...",
            value = "Initial value", width = 300, rows = 3, button = tags$button(id = "custom_submit",
              class = "btn btn-primary", "Send"), toolbar = tagList(tags$span("Press"),
            tags$kbd("Enter"), tags$span("to submit")), submit_key = "enter", spellcheck = "false",
            autocomplete = "off", )
        Output
Enter text
    Initial value
    # input_submit_textarea() validation errors

        Code
          input_submit_textarea("test", "With Children", div())
        Condition
          Error in `input_submit_textarea()`:
          ! All `...` arguments must be named
          i Did you mean to pass UI elements to `toolbar`?

    ---

        Code
          input_submit_textarea("test", value = 123)
        Condition
          Error in `input_submit_textarea()`:
          ! `value` must be a character string

    ---

        Code
          input_submit_textarea("test", value = c("a", "b"))
        Condition
          Error in `input_submit_textarea()`:
          ! `value` must be a character string

    ---

        Code
          input_submit_textarea("test", button = tags$div("Not a button"))
        Condition
          Error in `input_submit_textarea()`:
          ! `button` must be a `tags$button()`

    ---

        Code
          input_submit_textarea("test", submit_key = "invalid")
        Condition
          Error in `input_submit_textarea()`:
          ! `submit_key` must be one of "enter+modifier" or "enter", not "invalid".