# navbar markup snapshots Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 3))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 4))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 5))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 4), navbar_options = navbar_options( theme = "dark"))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 4), navbar_options = navbar_options( theme = "light"))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 4), navbar_options = navbar_options( bg = "#000"))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 5), navbar_options = navbar_options( theme = "dark"))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 5), navbar_options = navbar_options( theme = "light"))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 5), navbar_options = navbar_options( bg = "#000"))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 5), navbar_options = navbar_options( theme = "light", `data-bs-theme` = "dark"))) Output
--- Code show_navbar_markup(navs_bar_(theme = bs_theme(version = 5), navbar_options = navbar_options( class = "bg-primary", theme = "dark"))) Output