    # card_image()

        Code
          show_raw_html(card(card_image("https://example.com/image.jpg"), card_body(
            "image cap on top of card")))
        Output
image cap on top of card
    ---

        Code
          show_raw_html(card(card_body("image cap on bottom of card"), card_image(
            "https://example.com/image.jpg")))
        Output
image cap on bottom of card
    ---

        Code
          show_raw_html(card(card_header("header"), card_image(
            "https://example.com/image.jpg"), card_body("image not a cap")))
        Output
header
image not a cap
    ---

        Code
          show_raw_html(card(card_image("https://example.com/image.jpg", alt = "card-img")))
        Output
    # card_image() input validation

        Code
          card_image("cat.jpg")
        Condition
          Error in `card_image()`:
          ! `file` does not exist: cat.jpg
          i If `file` is a remote file or will be served by the Shiny app, use a URL or set `src = "cat.jpg"`.

    ---

        Code
          card_image("foo", "bar")
        Condition
          Error in `card_image()`:
          ! Unnamed arguments were included in `...`.
          i All additional arguments to `card_image()` in `...` should be named attributes for the `
` tag. --- Code card_image("foo", border_radius = "guess") Condition Error in `card_image()`: ! `border_radius` must be one of "auto", "top", "bottom", "all", or "none", not "guess".