# as_fill_item() with a simple tag Code cat(format(as_fill_item(tag_simple()))) Output
# as_fill_item() with a simple tag with arguments Code cat(format(ts_fill)) Output
# as_fill_item() with a nested tag on outer tag Code cat(format(as_fill_item(tag_nested()))) Output
# as_fill_item() with a nested tag on inner tag Code cat(format(nested_fill)) Output
# as_fillable_container() with a simple tag Code cat(format(as_fillable_container(tag_simple()))) Output
# as_fillable_container() with a simple tag with arguments Code cat(format(ts_fillable)) Output
# as_fillable_container() with a nested tag on outer tag Code cat(format(as_fillable_container(tag_nested()))) Output
# as_fillable_container() with a nested tag on inner tag Code cat(format(nested_fillable)) Output
# as_fill_carrier() with a simple tag Code cat(format(as_fill_carrier(tag_simple()))) Output
# as_fill_carrier() with a simple tag with arguments Code cat(format(ts_carrier)) Output
# as_fill_carrier() with a nested tag on outer tag Code cat(format(as_fill_carrier(tag_nested()))) Output
# as_fill_carrier() with a nested tag on inner tag Code cat(format(nested_carrier)) Output