# page_navbar() Code renderTags(page_navbar(title = div(h1("foo"), h2("bar"))))$head Output
--- Code renderTags(page_navbar(title = "foo", window_title = "bar"))$head Output bar
--- Code renderTags(page_sidebar(title = "foo"))$head Output foo
# page_sidebar() Code renderTags(page_sidebar(title = "foo", window_title = "bar"))$head Output bar
--- Code renderTags(page_sidebar("main", title = "Title", sidebar = sidebar(open = "always"), `data-attr` = "here"))$html Output
main
--- Code renderTags(page_sidebar("main", title = "Title", sidebar = "side"))$html Output
main