% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{deprecated}
\alias{deprecated}
\alias{bs_theme_new}
\alias{bs_theme_clear}
\alias{bs_theme_get}
\alias{bs_theme_set}
\alias{bs_theme_base_colors}
\alias{bs_theme_accent_colors}
\alias{bs_theme_fonts}
\alias{bs_theme_add_variables}
\alias{bs_theme_add}
\alias{bs_theme_get_variables}
\alias{bootstrap}
\alias{bootstrap_sass}
\alias{bs_add_declarations}
\alias{card_body_fill}
\alias{page_fill}
\alias{sidebar_toggle}
\alias{nav}
\alias{nav_content}
\alias{navs_tab}
\alias{navs_pill}
\alias{navs_pill_list}
\alias{navs_hidden}
\alias{navs_bar}
\alias{navs_tab_card}
\alias{navs_pill_card}
\title{Deprecated functions}
\usage{
bs_theme_new(...)

bs_theme_clear(...)

bs_theme_get(...)

bs_theme_set(...)

bs_theme_base_colors(...)

bs_theme_accent_colors(...)

bs_theme_fonts(...)

bs_theme_add_variables(...)

bs_theme_add(...)

bs_theme_get_variables(...)

bootstrap(theme = bs_theme_get(), ...)

bootstrap_sass(rules = list(), theme = bs_theme_get(), ...)

bs_add_declarations(theme, declarations)

card_body_fill(...)

page_fill(...)

sidebar_toggle(id, open = NULL, session = get_current_session())

nav(...)

nav_content(...)

navs_tab(...)

navs_pill(...)

navs_pill_list(...)

navs_hidden(...)

navs_bar(...)

navs_tab_card(...)

navs_pill_card(...)
}
\value{
a \code{\link[=bs_theme]{bs_theme()}} object.
}
\description{
These functions have been deprecated but remain for backwards compatibility.
}
\section{v0.9.1}{

The function \code{sidebar_toggle()} is now deprecated in v0.9.1. Please use
\code{\link[=toggle_sidebar]{toggle_sidebar()}} instead.
}

\section{Navigation Containers}{


Several functions for navigation containers were renamed in version 0.5.0:
\itemize{
\item \code{nav()} was renamed \code{\link[=nav_panel]{nav_panel()}}
\item \code{nav_content()} was renamed \code{\link[=nav_panel_hidden]{nav_panel_hidden()}}
\item \code{navs_tab()} was renamed \code{\link[=navset_tab]{navset_tab()}}
\item \code{navs_pill()} was renamed \code{\link[=navset_pill]{navset_pill()}}
\item \code{navs_pill_list()} was renamed \code{\link[=navset_pill_list]{navset_pill_list()}}
\item \code{navs_hidden()} was renamed \code{\link[=navset_hidden]{navset_hidden()}}
\item \code{navs_bar()} was renamed \code{\link[=navset_bar]{navset_bar()}}
\item \code{navs_tab_card()} was renamed \code{\link[=navset_card_tab]{navset_card_tab()}}
\item \code{navs_pill_card()} was renamed \code{\link[=navset_card_pill]{navset_card_pill()}}
}
}

\keyword{internal}
